#! perl
use warnings;
use strict;

use Test::More 'no_plan';

BEGIN { use_ok 'MyOpen' }
ok defined &CORE::GLOBAL::open, "CORE::GLOBAL::open() defined";

my $content;
{
    CORE::open( my $fh, '<', $0 ) or die "Cannot CORE::open($0): $!";
    isa_ok $fh, 'GLOB';
    $content = do { local $/; <$fh> };
    close $fh;
    like $content, qr/BEGIN { use_ok 'MyOpen' }/, "contains MyOpen";
}
use_ok 'FeedIn';
{
    no warnings 'redefine';
    local *CORE::GLOBAL::open = \&tied_open;
    open my $fh, '<', $0 or die "Cannot tied_open($0): $!";
    isa_ok tied( $fh ), 'FeedIn';
    my $file = do { local $/; <$fh> };
    close $fh;

    is $file, "open: $0", "tied_open() returned '$file'";
}

sub tied_open (*;$@) {
    my( $handle, $mode, $file ) = @_;

    # make sure filehandles are in their own package
    my $pkg = caller;
    if ( defined $handle and !ref $handle ) { # bareword handle
        no strict 'refs';
        $handle = *{ "$pkg\:\:$handle" };
    } elsif ( !defined $handle ) { # lexical undefined, provide a GLOB
        $_[0] = $handle = do {
            no strict 'refs';
            *{ sprintf "%s::NH%d%d%d", $pkg, $$, time, rand 100 };
        };
    }

    # convert to two argumented open()
    defined $file and $mode .= " $file";

    # do the magic-tie for open "< $0" or pass to CORE::open()
    if ( $mode =~ m/^(?:<\s*)?($0)/ ) {
        tie $handle, FeedIn => "open: $1";
    } else {
        CORE::open( $handle, $mode );
    }
}

# vim: syn=perl
