package Coords;
use warnings;
use strict;

use overload
    q{""}    => \&as_string,
    q{+}     => \&add_move,
    fallback => 1;

sub import {
    my $class = shift;
    if ( @_ and grep /^:constants$/ => @_ ) {
        overload::constant( 'q' => sub { __PACKAGE__->from_string( shift ) } );
    }
}

sub unimport {
    my $class = shift;
    if ( @_ and grep /^:constants$/ => @_ ) {
	overload::remove_constant( 'q' );
    }
}

sub new {
    my( $class, $x, $y ) = @_;
    if ( @_ == 2 ) {
        my @c = maybe_coords( $x );
        @c == 2 and ( $x, $y ) = @c;
    }
    bless { _x => $x || 0, _y => $y || 0 }, $class;
}

sub copy {  return bless { _x => $_[0]->{_x}, _y => $_[0]->{_y} }, ref $_[0]; }

sub move {
    my( $self, $dx, $dy ) = @_;
    ref $dx and ( $dx, $dy ) = ( $dx->{_x}, $dx->{_y} );

    $self->{_x} += $dx;
    $self->{_y} += $dy;
    return $self;
}

sub add_move {
    my( $a1, $a2, $rev ) = @_;
    ref $a2 or $a2 = __PACKAGE__->from_string( $a2 );
    ref $a2 or die "Cannot convert '$a2' to Coords";
    $rev and ( $a1, $a2 ) = ( $a2, $a1 );
    $a1->copy->move( $a2 );
}

sub as_string {
    my( $self ) = @_;
    return sprintf "(%d, %d)", $self->{_x}, $self->{_y};
}

sub from_string {
    my $class_or_instance = shift;
    my $string = shift;
    if ( my @c = maybe_coords( $string ) ) {
        return __PACKAGE__->new( @c );
    } else {
        return $string;
    }
}

sub maybe_coords {
    $_[0] =~ /^\(\s*(-?\d+)\s*,\s*(-?\d+)\s*\)$/;
}

1;

__END__

=head1 NAME

Coords - An OO module to manipulate a set of two numebers (coordinates)

=head1 SYNOPSIS

    use Coords;

    my $c1 = Coords->new( 150, 150 );
    my $c2 = Coords->new( "(50, -50)" );

    my $r1 = $c1;
    $r1->move( $c2 );

    print "After the move: $r1\n"; # (200, 100)

=head1 DESCRIPTION

=cut
