package Hugeint::ByteArray;
use warnings;
use strict;

our $VERSION = 0.01;

sub TIEARRAY {
    my( $class, @init ) = @_;
    my $self = bless \(my $store), $class;
    if ( @init ) { $self->PUSH( @init ) } else { $$self = '' }
    return $self;
}

sub FETCH {
    my( $self, $index ) = @_;
    $index >= length $$self and $self->_grow( $index );
    ord( substr $$self, $index, 1 );
}

sub STORE {
    my( $self, $index, $value ) = @_;
    $index >= length $$self and $self->_grow( $index );
    substr $$self, $index, 1, chr( $value );
}

sub FETCHSIZE { length ${ $_[0] } }

sub STORESIZE {
    my( $self, $size ) = @_;
    if ( $size > length $$self ) {
        $self->_grow( $size );
    } else {
        $$self = substr $$self, 0, $size;
    }
    $size;
}

sub PUSH {
    my( $self, @values ) = @_;
    $$self .= chr( $_ ) for @values;
}

sub POP {
    my( $self ) = @_;
    my $last = substr $$self, -1, 1;
    $$self = substr $$self, 0, length( $$self ) - 1;
    return ord $last;
}

sub SHIFT {
    my( $self ) = @_;
    my $first = substr $$self, 0, 1;
    $$self = substr $$self, 1;
    return ord $first
}

sub UNSHIFT {
    my( $self, @values ) = @_;
    my $insert = join "", map chr( $_ ) => @values;
    substr $$self, 0, 0, $insert;
}

sub CLEAR { ${ $_[0] } = "" }

sub EXTEND { }

sub _grow {
    my( $self, $size ) = @_;
    $size >= length $$self and $$self .= "\00" x (1 + $size - length $$self);
}

1;

