#! perl
use warnings;
use strict;

use Test::More tests => 53;

use_ok 'Hugeint';

my $x = Hugeint->new( 10 );
isa_ok $x, 'Hugeint';
is "$x", "10", "stringoverloading works";

my $y = $x + 10;
isa_ok $y, 'Hugeint', "after add-overloading";
is $y, "20", "addition $y";

my $z1 = $y - $x;
isa_ok $z1, "Hugeint", "after overloaded arithmatic";
is $z1, 10, "$y - $x = $z1";

my $z2 = -$x + $y;
isa_ok $z2, "Hugeint", "after overloaded arithmatic";
is $z2, 10, "- $x + $y = $z2";

$z1->assign( 1 );
is ~$z1, -2, "bitwise negation of 1 == -2";

$z1 *= 15;
is $z1, 15, "*= 15 -> $z1";

$z1 /= 3;
is $z1, 5, "/= 3 -> $z1";

$z1 **= 3;
is $z1, 125, "**= 3 -> $z1";

$z1 = $x ** 5;
is $x, 10, "argument unchanged";
is $z1, 100000, "10^5 = $z1";

$z2 = $z1 / 1000;
is $z2, 100, "100000/1000 = $z2";

$z2 = $z2 % 3;
is $z2, 1, "100 % 3 = $z2";

$z2 += 4;
is $z2, 5, "1 += 4 -> $z2";

$z2 -= 2;
is $z2, 3, "5 -= 2 -> 3";

$z2 = $z2 * 5;
is $z2, 15, "3 * 5 = $z2";

for my $i ( 0..1 ) {
    if ( $i % 2 == 0 ) {
        is $x <=> $y, -1, "<=>: $x < $y";
    } else {
        is $y <=> $x, 1, "<=>: $y > $x";
    }
}
is $x <=> $x, 0, "<=>: $x == $x";
ok $x == $x, "== $x == $x";

$x->negate; # x-; y+
for my $i ( 0..1 ) {
    if ( $i % 2 == 0 ) {
        is $x <=> $y, -1, "<=>: $x < $y";
    } else {
        is $y <=> $x, 1, "<=>: $y > $x";
    }
}
is $x <=> $x, 0, "<=>: $x == $x";
ok $x == $x, "== $x == $x";

$x->negate; $y->negate; # x+; y-
for my $i ( 0..1 ) {
    if ( $i % 2 == 0 ) {
        is $x <=> $y, 1, "<=>: $x > $y";
    } else {
        is $y <=> $x, -1, "<=>: $y < $x";
    }
}
is $x <=> $x, 0, "<=>: $x == $x";
ok $x == $x, "== $x == $x";

$x->negate; # x-; y-
for my $i ( 0..1 ) {
    if ( $i % 2 == 0 ) {
        is $x <=> $y, 1, "<=>: $x > $y";
    } else {
        is $y <=> $x, -1, "<=>: $y < $x";
    }
}
is $x <=> $x, 0, "<=>: $x == $x";
ok $x == $x, "== $x == $x";
$x->negate; $y->negate; # x+; y+

is 20 <=> $x, 1, "<=>: 20 > hi($x)";

$z1->assign( 0 );
ok !$z1, "boolean negation @{[ $z1->is_zero ]}";

$z1 |= 255;
is $z1, 255, "0 | 255 = $z1";

$z1 %= 4;
is $z1, 3, "255 %= 4 -> $z1";

$z1 &= 255;
is $z1, 3, "3 &= 255 -> $z1";

{
    my $z = $z1;
    isa_ok $z, 'Hugeint', "after 0+";
    is $z, 3, "value copied: $z";
}
{
    my $z = 10 + $x;
    isa_ok $z, 'Hugeint';
    is $z, 20, "10 + hi(10) = $z";
}
{
    my $z = $x;
    isa_ok $z, 'Hugeint';
    is $z, 10, "copy $z";

    $z++;
    is $z, 11, "++ -> $z";

    $z--;
    is $z, 10, "-- -> $z"
}
{
    my $z = Hugeint->new( 7 );
    isa_ok $z, 'Hugeint';

    $z &= 255;
    is $z, 7, "7 & 255 = $z";

    $z |= 248;
    is $z, 255, "7 | 255 = $z";

    $z ^= 7;
    is $z, 248, "255 ^ 7 = $z";
}
